//+------------------------------------------------------------------+
//|                                          LinearReg_ChannelEA.mq5 |
//|                        Copyright 2019, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2019, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#resource  "\\Indicators\\LRL-Mod.ex5"
#resource  "\\Indicators\\LinregChannel.ex5"

#include  <Trade\Trade.mqh>

enum DIRECTION
  {
   DIRECTION_BUY,
   DIRECTION_SELL,
   DIRECTION_BUYSELL
  };
enum INDICATOR
  {
   INDICATOR_LRL_MOD ,
   INDICATOR_LINREG_NEW
  };
input group "=========== Linear Regression Indicator ========"
input INDICATOR _Indicator=INDICATOR_LRL_MOD ; // Indicator in use
input int      LRMAPeriod=14; // Period
input ENUM_APPLIED_PRICE _Price=PRICE_CLOSE ; // Price
input int   _Offset=0 ; // Offset
input int   _Gap=100 ; // GAP Points
input ENUM_TIMEFRAMES _TimeFrame=PERIOD_CURRENT ; // Time Frame

input group "=========== Order Settings ========"
input DIRECTION _Direction=DIRECTION_BUYSELL ; // Direction
input bool     _ReverseSignal=false ; // Reverse Signal
input string  _StartTime="00:00" ; // Time Start
input string  _EndTime="23:00" ; // Time End
input   double    _LotSize=0.10 ;
input bool  _UseFixedTPSL=true ; // Use Fixed TP & SL
input int   _FixedTP=300 ; // Fixed TP Points
input int   _FixedSL=300 ; // Fixed SL Points
input int   _Magic=1234 ; // Magic Number

int linregHandle=INVALID_HANDLE ;
int bars=0;
CTrade Trade ;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   linregHandle=_Indicator==INDICATOR_LRL_MOD ?iCustom(_Symbol,_TimeFrame,"::Indicators\\LRL-Mod.ex5",LRMAPeriod,_Price,_Offset,_Gap):
                                               iCustom(_Symbol,_TimeFrame,"::Indicators\\LinregChannel.ex5",LRMAPeriod,_Price,_Offset,_Gap) ;
   if(linregHandle==INVALID_HANDLE)
     {
      Print("Can't Get Indicator") ;
      return INIT_FAILED ;
     }
     Trade.SetExpertMagicNumber(_Magic) ;
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---

  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   int counted=BarsCalculated(linregHandle);
   if(counted<=0)
     {
      Print("Data not ready ") ;
      return ;
     }
   
   double lrl[] , lru[] ,close[] ,open[],high[],low[];
   ArraySetAsSeries(lrl,true);
   ArraySetAsSeries(lru,true) ;
   ArraySetAsSeries(close,true);
   int cnt1=CopyBuffer(linregHandle,1,1,1,lru);
   int cnt2=CopyBuffer(linregHandle,2,1,1,lrl);
   int cnt3=CopyClose(_Symbol,_TimeFrame,1,1,close) ;

   
   if(!IsTradingTime())
    {
      for(int i=PositionsTotal()-1;i>=0;i--)
        {
          ulong tkt=PositionGetTicket(i) ;
          if(tkt<=0) continue ;
          string symbol=PositionGetSymbol(i);
          if(symbol!=_Symbol)continue ;
          int magic=(int)PositionGetInteger(POSITION_MAGIC) ;
          if(magic!=_Magic)return ;
          if(!Trade.PositionClose(tkt,300))
           {
            Print("Can't Close Position , reason : " , Trade.CheckResultRetcodeDescription()) ;
           }
        }
        return ;
    }
    
   if(GetTotalPositionsCount()==0)
    {
      int takeprofit_point =_UseFixedTPSL ? _FixedTP : _Gap ;
      int stoploss_point=_UseFixedTPSL ? _FixedSL : _Gap ;
      double Ask=SymbolInfoDouble(_Symbol,SYMBOL_ASK);
      double Bid=SymbolInfoDouble(_Symbol,SYMBOL_BID);
      double tickSize=SymbolInfoDouble(_Symbol,SYMBOL_TRADE_TICK_SIZE) ;
      int stop_level=(int)SymbolInfoInteger(_Symbol,SYMBOL_TRADE_STOPS_LEVEL) ;
      
      if( (takeprofit_point>0 && takeprofit_point<stop_level) || (stoploss_point>0 && stoploss_point<stop_level))
       {
        Print("Invalid Stop Loss or Take Profit Points , Less than minimum required") ;
        return ;
       }
     if(_Direction!=DIRECTION_SELL && ((!_ReverseSignal &&close[0]<=lrl[0])||(_ReverseSignal && close[0]>=lru[0])) && bars!=Bars(_Symbol,_TimeFrame))
      {
       
       double tp=takeprofit_point<=0 ? 0 : NormalizeDouble(Ask+takeprofit_point* tickSize,_Digits) ;
       double sl=stoploss_point<=0 ? 0 : NormalizeDouble(Ask-stoploss_point*tickSize , _Digits ) ;
       if(!Trade.Buy(_LotSize,_Symbol,Ask,sl,tp,""))
        {
         Print("Can't Send Buy Order , reason " ,Trade.CheckResultRetcodeDescription());
        }
       else
        {
        bars=Bars(_Symbol,_TimeFrame);
        }
      }
     else if(_Direction!=DIRECTION_BUY && ((!_ReverseSignal && close[0]>=lru[0])||(_ReverseSignal && close[0]<=lrl[0])) && bars!=Bars(_Symbol,_TimeFrame))
      {
        double tp=takeprofit_point<=0 ? 0 : NormalizeDouble(Bid-takeprofit_point* tickSize,_Digits) ;
         double sl=stoploss_point<=0 ? 0 : NormalizeDouble(Bid+stoploss_point*tickSize , _Digits ) ;
         if(!Trade.Sell(_LotSize,_Symbol,Bid,sl,tp,""))
        {
         Print("Can't Send Sell Order , reason " ,Trade.CheckResultRetcodeDescription());
        }
        else
        {
         bars=Bars(_Symbol,_TimeFrame);
        }
      }
    }

  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool IsTradingTime()
  {
   if(_StartTime=="" || _EndTime=="")
      return true;

   datetime timeStart=StringToTime(TimeToString(TimeCurrent(),TIME_DATE)+" "+_StartTime);
   datetime timeEnd=StringToTime(TimeToString(TimeCurrent(),TIME_DATE)+" "+_EndTime);

   if(timeStart>=timeEnd)
     {
      if(TimeCurrent()>timeEnd)
        {
         timeEnd+=PeriodSeconds(PERIOD_D1);
        }
      else
         if(TimeCurrent()<=timeEnd)
           {
            timeStart-=PeriodSeconds(PERIOD_D1);
           }
     }

   if(TimeCurrent()>=timeStart && TimeCurrent()<=timeEnd)
      return true;
   return false;
  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int GetTotalPositionsCount()
  {
   int cnt=0;
   for(int i=0; i<PositionsTotal(); i++)
     {
      ulong tkt=PositionGetTicket(i);
      if(tkt>0)
        {
         string symbol=PositionGetSymbol(i);
         int magic=(int)PositionGetInteger(POSITION_MAGIC);
         ENUM_POSITION_TYPE ptype=(ENUM_POSITION_TYPE) PositionGetInteger(POSITION_TYPE);
         if(symbol==_Symbol && magic==_Magic)
            cnt++;
        }
     }
   return cnt;
  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int GetTotalPositionsCount(ENUM_POSITION_TYPE type)
  {
   int cnt=0;
   for(int i=0; i<PositionsTotal(); i++)
     {
      ulong tkt=PositionGetTicket(i);
      if(tkt>0)
        {
         string symbol=PositionGetSymbol(i);
         int magic=(int)PositionGetInteger(POSITION_MAGIC);
         ENUM_POSITION_TYPE ptype=(ENUM_POSITION_TYPE) PositionGetInteger(POSITION_TYPE);

         if(symbol==_Symbol && magic==_Magic && ptype==type)
            cnt++;
        }
     }
   return cnt;
  }
//+------------------------------------------------------------------+
